@@ -24,7 +24,6 @@ from django.http import (
 )
 from django.test import SimpleTestCase
 from django.utils.functional import lazystr
-from django.utils.http import MAX_URL_LENGTH
 
 
 class QueryDictTests(SimpleTestCase):
@@ -353,9 +352,9 @@ class HttpResponseTests(SimpleTestCase):
         h.headers["Content-Disposition"] = 'attachment; filename="%s"' % f
         # This one is triggering https://bugs.python.org/issue20747, that is Python
         # will itself insert a newline in the header
-        h.headers[
-            "Content-Disposition"
-        ] = 'attachment; filename="EdelRot_Blu\u0308te (3)-0.JPG"'
+        h.headers["Content-Disposition"] = (
+            'attachment; filename="EdelRot_Blu\u0308te (3)-0.JPG"'
+        )
 
     def test_newlines_in_headers(self):
         # Bug #10188: Do not allow newlines in headers (CR or LF)
@@ -491,7 +490,6 @@ class HttpResponseTests(SimpleTestCase):
             'data:text/html,<script>window.alert("xss")</script>',
             "mailto:test@example.com",
             "file:///etc/passwd",
-            "é" * (MAX_URL_LENGTH + 1),
         ]
         for url in bad_urls:
             with self.assertRaises(DisallowedRedirect):
@@ -756,7 +754,7 @@ class StreamingHttpResponseTests(SimpleTestCase):
             "serve them asynchronously. Use an asynchronous iterator instead."
         )
         with self.assertWarnsMessage(Warning, msg):
-            self.assertEqual(b"hello", await r.__aiter__().__anext__())
+            self.assertEqual(b"hello", await anext(aiter(r)))
 
 
 class FileCloseTests(SimpleTestCase):
@@ -764,9 +762,7 @@ class FileCloseTests(SimpleTestCase):
         # Disable the request_finished signal during this test
         # to avoid interfering with the database connection.
         request_finished.disconnect(close_old_connections)
-
-    def tearDown(self):
-        request_finished.connect(close_old_connections)
+        self.addCleanup(request_finished.connect, close_old_connections)
 
     def test_response(self):
         filename = os.path.join(os.path.dirname(__file__), "abc.txt")
